#!/bin/bash
# Script copias de seguridad en local
# Solo se permite la copia en el disco indicado
# Realizará el montaje del dispositivo incluso si está montado (desmontando previamente)
# Opcional. Si se proporcinonan credenciales, podría enviar una copia al correo del programa
# informando de lo que ha ocurrido en el caso de ejecutarlo en modo no interactivo
#  Para ello se necesita configurar los parámentros SMTP del fichero de configuración:
#   /etc/default/vx-dga-variables/backup.conf
# Por útimo recordar descomentar el fichero CRON si queremos periodicidad

CONFFILE="/etc/default/vx-dga-variables/backup.conf"

notifiy_mail (){
    local SERVER_PORT
    local SENDER
    local USER
    local PASSWORD
    local RECEIVER="$1"
    local RESULT="$2"
    local SUBJECT="$3"
    local ATTACH="$4"
    
    SERVER_PORT=$(crudini --get ${CONFFILE} SMTP servermailpor) || return 1
    SENDER=$(crudini --get ${CONFFILE} SMTP srcemail) || return 1
    USER="$SENDER"
    PASSWORD=$(crudini --get ${CONFFILE} SMTP srcpass) || return 1
    ATTACHTYPE=$(file -b --mime-type "$ATTACH")
    
    # Mandamos el log del envío de correo a un fichero temporal
    TMPFILE=$(mktemp --tmpdir send_mail.XXXXX)
    exec 6>&1
    exec >"$TMPFILE" 2>&1
    
    if [ "$RESULT" = "OK" ]; then
        DATOS="El proceso de Backup se ha llevado a cabo correctamente el $(date)\n\
        Se adjunta fichero de trazas de la copia realizada\n\n\
        Autor: Responsables Técnicos del Programa Vitalinux DGA"
        elif [ "$RESULT" = "FAIL" ]; then
        DATOS="El proceso de Backup NO se ha llevado a cabo correctamente el $(date)\n\
        Se adjunta fichero de trazas de la copia realizada\n\n\
        Autor: Responsables Técnicos del Programa Vitalinux DGA\n"
    else
        DATOS=$RESULT
    fi
    echo "**** $(date) - Inicio de envío de mensaje ********"
    swaks --to "$RECEIVER" --from "$SENDER" --server "$SERVER_PORT" --auth LOGIN --auth-user "$USER" --auth-password "$PASSWORD" -tls \
    --body "$DATOS" --add-header "MIME-Version: 1.0" --header "Subject: $SUBJECT" \
    --attach-type "$ATTACHTYPE" --attach "$ATTACH"
    [ "$?" = "0" ] && echo "Correo Mandado"
    echo "**** $(date) - FIN Transmisión de mensaje ********"
    
    # Cerramos el descriptor 1 actual -> fichero y recuperamos salida estandar
    exec 1>&- &&  exec 1>&6 && exec 2>&6 && exec 6>&-
    
    
}

close_log_send_file (){
    # Por si acaso, debermos cerrar el fichero antes
    if [ -n "$LOGFILE" ] ; then
        # Si el descriptor está ya cerrado para escritura, entonces no hago nada y salgo de la función
        true 2>/dev/null >&6 || return 0
        
        # Sino, cerramos el desriptor 1 que apuntaba al fichero...para cerrarlo
        exec 1>&-
        # Ahora usamos el 6 para recuperar la salidas estandar/error originales
        exec 1>&6
        exec 2>&6
        # Y cerramos el 6 usado
        exec 6>&-
        
        # Ahora mandamos el correo si se ejecuta en modo no interactivo (en caso interactivo no merece la pena no?)
        if [ "$STATUS" = "OK" ] && [ "$MAILING" = "1" ] ; then
            notifiy_mail "$DESTMAIL" "OK" "Copia de seguridad LOCAL OK" "$LOGFILE" || echo "Error al enviar la notificación. Revisar parámetros y conexión"
            elif [ "$MAILING" = "1" ]; then
            notifiy_mail "$DESTMAIL" "Copia de seguridad LOCAL FAIL" "FAIL" "$LOGFILE" || echo "Error al enviar la notificación. Revisar parámetros y conexión"
        fi
    fi
    
}

usage (){
    echo "Copia de seguridad de migasfree proyecto en Local"
    echo "Modo de Uso: backup_en_cache.sh [-o LOG]"
    echo "Opciones posibles:"
    echo "   -o -> Para llamadas no interactivas, la información sobre el proceso de copia se volcará en el fichero LOG"
    echo "Al finalizar, si está en modo interactivo intentará mandar un mensaje por correo"
}
# Comprobamos opciones para redireccionar la salida

while getopts ':ho:' OPT; do
    case $OPT in
        o)
            LOGFILE=$OPTARG
        ;;
        h)
            usage
            exit 0
        ;;
        \?)
            echo "Opción inválida: -$OPTARG" >&2
            exit 1
        ;;
        :)
            echo "Opción -$OPTARG requiere un argumento" >&2
            exit 1
        ;;
    esac
done

if [ -n "$LOGFILE" ] ; then
    ## Vamos a redireccionar las salidas pero antes de terminar querremos mandar el fichero generado por correo
    ## Por lo que es conveniente cerrarlo antes de enviar.
    ## Para ello nos guardamos el Descriptor de Fichero de la salida estandar en el 6 (libre) para recuperar
    ## la salida estandar al final (en el caso de lanzarlo el cron no haría falta, ya que no necesitamos que salga nada por pantalla,
    ## pero si cuando se hace
    ## en modo interactivo
    exec 6>&1
    ## Y redireccionamos todas las salidas al fichero
    exec &>> "$LOGFILE"
    trap 'close_log_send_file' INT TERM ERR EXIT
    # Ojo, con el trap anterior, cuando termine el proceso de forma normal también intentará cerrar el archivo...controlar si está abierto en la función
fi
STATUS="FAIL"
# Lo primero, comprobamos que está activo el servicio, según el fichero de configuración.
ACTIVO=$(crudini --get ${CONFFILE} Global activo) || { echo "No está definido el parámetro activo. Saliendo..."; exit 1; }
[ "$ACTIVO" = "0" ] && { echo "No está activado el servicio...debes configurarlo y activarlo en el fichero de configuración"; exit 1; }
#
UUIDVITA=$(crudini --get ${CONFFILE} Settings uuiddisk) || { echo "No está definido el uuid del disco. Saliendo..."; exit 1; }
DIRSCOPIAR=$(crudini --get ${CONFFILE} Settings directorios | tr ',' ' ') || { echo "No están definidos los directorios de copia. Saliendo..."; exit 1; }
ARRDIRSCOPIAR=($DIRSCOPIAR)
DIRSEXCLUIR=$(crudini --get ${CONFFILE} Settings excluir | tr ',' ' ') || DIRSEXCLUIR=""
DIRMOUNT="/mnt"
DIRDISCO="COPIASERVIDORCACHE"
MAILING="$(crudini --get ${CONFFILE} SMTP activo)" || MAILING="0"
if [ "$MAILING" != "0" ]; then
    DESTMAIL="$(crudini --get ${CONFFILE} SMTP dstmail)" || { echo "No hay destinatario definido en el envio"; exit 1; }
fi
DIRCOPIA="$DIRMOUNT/$DIRDISCO"
DIRDELETEFILES="$DIRMOUNT/DELETEFILES"

if grep -e " $DIRMOUNT " /proc/mounts; then
    if ! umount /mnt; then
        echo "Directorio de montaje $DIRMOUNT ocupado...desmontar previamente"
        exit 1
    fi
fi
if ! mount UUID="$UUIDVITA" /mnt/; then
    echo "No se puede montar el disco de Copias de Seguridad: $UUIDVITA"
    exit 1
fi
[ ! -d $DIRCOPIA ] && mkdir -p $DIRCOPIA
[ ! -d $DIRDELETEFILES ] && mkdir -p $DIRDELETEFILES

echo "#######################################"
echo "Copia de seguridad de $(date +%Y-%m-%d)"

STATUS="OK"
# Crearmos la linea de directorios a excluir
declare -a ARREXCLUDE
for EXC in $DIRSEXCLUIR; do
    ARREXCLUDE+=("--exclude=$EXC")
done
echo " Sincornizando los directorios: #$DIRSCOPIAR#"
echo " Excluir: ${ARREXCLUDE[*]}"
echo " Destino: $DIRCOPIA"
echo " Directorio de Ficheros borrados en el destino: $DIRDELETEFILES"
if rsync -av "${ARREXCLUDE[@]}" --delete --backup --backup-dir="${DIRDELETEFILES}"/"$(date "+%Y_%m_%d-%H_%M_%S")" "${ARRDIRSCOPIAR[@]}" "$DIRCOPIA" ; then
    echo "Backup Realizado correctamente de $DIRSCOPIAR"
else
    echo "Hubo un error al sincronizar localmente $DIRSCOPIAR "
    STATUS="FAIL"
fi
echo "#######################################"
echo "############## FIN ####################"


# Cuando termine el proceso, si estamos en modo no interactivo como capturamos el EXIT
#  se ejecutará la función de salir definida (que cierra y envía el correo)
if ! umount /mnt; then
    echo "ATENCIÓN: No se puede desmontar el disco de copias...requiere revisión manual"
    exit 1
fi
exit 0
